package com.example.gajz.transferrelationship.controller;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;
import com.alibaba.fastjson.JSON;
import com.example.gajz.result.Result;
import com.example.gajz.result.ResultCode;
import com.example.gajz.transferrelationship.bean.excel.abcExcie;
import com.example.gajz.transferrelationship.bean.param.DiagramSpotParam;
import com.example.gajz.transferrelationship.bean.param.TransferAssociationParam;
import com.example.gajz.transferrelationship.bean.vo.EchartsUserVo;
import com.example.gajz.transferrelationship.bean.vo.SearchRelationshipVo;
import com.example.gajz.transferrelationship.service.TransferRelationshipService;
import org.apache.poi.openxml4j.util.ZipSecureFile;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
 * @author lenovo
 */
/**跨域注解*/
@CrossOrigin(origins = "*")
@Controller
public class TransferRelationshipController {

    @Autowired
    private TransferRelationshipService transferRelationshipService;

    /**
     * 转账关系
     */
    @ResponseBody
    @RequestMapping("edge")
    private List<com.example.gajz.transferrelationship.bean.param.data> edge(){
        List<com.example.gajz.transferrelationship.bean.param.data> edge = transferRelationshipService.edge();
        return edge;
    }

    /**
     * 点
     * numerical 参数
     * 传0不重新刷新  数据从Redis里面取出
     * 传1重新刷新    数据从数据库里取出  并且重新生成位置
     * drawingNumber 参数
     * 传数字 是代表对应的各种图形算法
     */
    @ResponseBody
    @RequestMapping("/diagramSpot")
    private List diagramSpot(DiagramSpotParam diagramSpotParam){
        List list = transferRelationshipService.diagramSpot(diagramSpotParam);
        return list;
    }


    /**
     * 关系图 搜索接口
     * @param name
     * @return
     */
    @ResponseBody
    @RequestMapping("/pointSearch")
    private Result pointSearch(String name){
        Result result = transferRelationshipService.pointSearch(name);
        return result;
    }

    /**
     * 关系图搜索 单人转账关系（关联关系） （点）接口
     * @param cardnumbe
     * @return
     */
    @ResponseBody
    @RequestMapping("/searchRelationship")
    private Result searchRelationship(String cardnumbe){
        Result result = transferRelationshipService.searchRelationship(cardnumbe);
        return result;
    }

       
    /**
     * 关系图搜索  单人转账关系    转账信息信息查询接口
     * @return
     */
    @ResponseBody
    @RequestMapping("/personalTransferRelationship")
    public Result personalTransferRelationship(int RedisId){
        Result result = transferRelationshipService.personalTransferRelationship(RedisId);
        return result;
    }



    /**
     *  关系图搜索  点击查询某一个点的具体位置
     */
    @ResponseBody
    @RequestMapping("/specificLocation")
    private Result specificLocation(@RequestParam("id")String id){
        Result result = transferRelationshipService.specificLocation(id);
        return result;
    }


    /**
     * 查询两人转账关系  点  (关系图) 接口
     * @param transferAssociationParam
     * @return
     */
    @ResponseBody
    @RequestMapping("/transferAssociation")
    private Result transferAssociation(TransferAssociationParam transferAssociationParam){
        Result result = transferRelationshipService.transferAssociation(transferAssociationParam);
        return result;
    }

    /**
     * 查询两人转账关系  关系  (关系图) 接口
     * @param transferAssociationParam
     * @return
     */
    @ResponseBody
    @RequestMapping("/trBetweenTwoPersons")
    private Result trBetweenTwoPersons(@RequestBody List<SearchRelationshipVo> transferAssociationParam){
        Result result = transferRelationshipService.trBetweenTwoPersons(transferAssociationParam);
        return result;
    }


    /**
     * echarts 双人转账关系图 点  生成接口
     * @param transferAssociationParam
     * @return
     */
    @ResponseBody
    @RequestMapping("/echartsTransferAssociation")
    private Result echartsTransferAssociation(TransferAssociationParam transferAssociationParam){
        Result result = transferRelationshipService.echartsTransferAssociation(transferAssociationParam);
        return result;
    }

    /**
     * echarts 双人转账关系图 关系 生成接口
     * @param transferAssociationParam
     * @return
     */
    @ResponseBody
    @RequestMapping("/echartsTrBetweenTwoPersons")
    private Result echartsTrBetweenTwoPersons(String transferAssociationParam,String transfer){
        List<EchartsUserVo> userList= JSON.parseArray(transferAssociationParam,EchartsUserVo.class);
        List<SearchRelationshipVo> searchRelationshipVoList=new ArrayList<>();
        for (int i=0;i<userList.size();i++){
            SearchRelationshipVo searchRelationshipVo=new SearchRelationshipVo();
            searchRelationshipVo.setCardNumber(userList.get(i).getName());
            searchRelationshipVoList.add(searchRelationshipVo);
        }
        Result result = transferRelationshipService.echartsTrBetweenTwoPersons(searchRelationshipVoList);
        return result;
    }



    /**
     * 关系导入
     */
    @ResponseBody
    @RequestMapping("/transferRelationshipFile")
    private Result  saveSerialNo(MultipartFile file,int Accumulate) throws IOException {
        //判断是否清空之前数据  0是清空  1是累加
        if (Accumulate==0){
            //清空关系图数据表
            transferRelationshipService.emptyTransferRelationship();
        }
        /**
         * 存储员工对象
         */
        List<abcExcie> eSerialsList = new ArrayList<>();
        ZipSecureFile.setMinInflateRatio(-1.0d);
        EasyExcel.read(file.getInputStream(),
                abcExcie.class,
                new AnalysisEventListener<abcExcie>() {
                    //easyExcel每次从Excel中读取一行数据就会调用一次invoke方法
                    @Override
                    public void invoke(abcExcie serialEntity, AnalysisContext analysisContext) {
                        abcExcie abcexcie = new abcExcie();
                        //id
                        abcexcie.setId(serialEntity.getId());
                        //身份证号
                        abcexcie.setId_card(serialEntity.getId_card());
                        //手机号
                        abcexcie.setMobile(serialEntity.getMobile());
                        //本端账户名
                        abcexcie.setBank_account_name(serialEntity.getBank_account_name());
                        //本端账号
                        abcexcie.setBank_account_no(serialEntity.getBank_account_no());
                        //本端卡号
                        abcexcie.setBank_card_no(serialEntity.getBank_card_no());
                        //曾用卡号
                        abcexcie.setTemp_a(serialEntity.getTemp_a());
                        //本端账户开户行
                        abcexcie.setBank_name(serialEntity.getBank_name());
                        //是否属资金池,0是1否
                        abcexcie.setIs_capital_pool(serialEntity.getIs_capital_pool());
                        //交易时间
                        abcexcie.setTrade_date(serialEntity.getTrade_date());
                        //记账时间
                        abcexcie.setTrade_time(serialEntity.getTrade_time());
                        //收付标志，0收1支
                        abcexcie.setPayment_mark(serialEntity.getPayment_mark());
                        //借方发生额（支出）
                        abcexcie.setTemp_c(serialEntity.getTemp_c());
                        //贷方发生额（收入）
                        abcexcie.setTemp_d(serialEntity.getTemp_d());
                        //交易金额
                        abcexcie.setPay_amount(serialEntity.getPay_amount());
                        //余额
                        abcexcie.setBalance(serialEntity.getBalance());
                        //目的、用途
                        abcexcie.setPurpose(serialEntity.getPurpose());
                        //分红性质类别
                        abcexcie.setBonus(serialEntity.getBonus());
                        //其他交易性质
                        abcexcie.setTrade_nature(serialEntity.getTrade_nature());
                        //审计备注列1
                        abcexcie.setTemp_f(serialEntity.getTemp_f());
                        //审计备注列2
                        abcexcie.setTemp_g(serialEntity.getTemp_g());
                        //对端户名
                        abcexcie.setEnd_account_name(serialEntity.getEnd_account_name());
                        //对端卡号
                        abcexcie.setEnd_card_no(serialEntity.getEnd_card_no());
                        //对端开户行
                        abcexcie.setEnd_bank_name(serialEntity.getEnd_bank_name());
                        //交易摘要
                        abcexcie.setRemark(serialEntity.getRemark());
                        //交易备注
                        abcexcie.setComment(serialEntity.getComment());
                        //交易场所
                        abcexcie.setTreade_place(serialEntity.getTreade_place());
                        eSerialsList.add(abcexcie);
                    }
                    //easyExcel在将Excel表中数据读取完毕后，最终执行此方法
                    @Override
                    public void doAfterAllAnalysed(AnalysisContext analysisContext) {
                        int t=0;
                        int quantityPerServing =1000; //每份1w次插入数据
                        for (int s=0;t<=eSerialsList.size();){
                                List<abcExcie> eSerialsList1 = new ArrayList<>();
                                for (int i=0;t<quantityPerServing;t++){
                                    if (t<eSerialsList.size()){
                                        abcExcie abcExcie=new abcExcie();
                                        //身份证号
                                        abcExcie.setId_card(eSerialsList.get(t).getId_card());
                                        //手机号
                                        abcExcie.setMobile(eSerialsList.get(t).getMobile());
                                        //本端账户名
                                        abcExcie.setBank_account_name(eSerialsList.get(t).getBank_account_name());
                                        //本端账号
                                        abcExcie.setBank_account_no(eSerialsList.get(t).getBank_account_no());
                                        //本端卡号
                                        abcExcie.setBank_card_no(eSerialsList.get(t).getBank_card_no());
                                        //曾用卡号
                                        abcExcie.setTemp_a(eSerialsList.get(t).getTemp_a());
                                        //本端账户开户行
                                        abcExcie.setBank_name(eSerialsList.get(t).getBank_name());
                                        //是否属资金池,0是1否
                                        abcExcie.setIs_capital_pool(eSerialsList.get(t).getIs_capital_pool());
                                        //交易时间
                                        abcExcie.setTrade_date(eSerialsList.get(t).getTrade_date());
                                        //记账时间
                                        if (!"1899/12/31 0:00:00".equals(eSerialsList.get(t).getTrade_time())){
                                            abcExcie.setTrade_time(eSerialsList.get(t).getTrade_time());
                                        }else {
                                            abcExcie.setTrade_time(null);
                                        }
                                        //收付标志，0收1支
                                        abcExcie.setPayment_mark(eSerialsList.get(t).getPayment_mark());
                                        //借方发生额（支出）
                                        abcExcie.setTemp_c(eSerialsList.get(t).getTemp_c());
                                        //贷方发生额（收入）
                                        abcExcie.setTemp_d(eSerialsList.get(t).getTemp_d());
                                        //交易金额
                                        abcExcie.setPay_amount(eSerialsList.get(t).getPay_amount());
                                        //余额
                                        abcExcie.setBalance(eSerialsList.get(t).getBalance());
                                        //目的、用途
                                        abcExcie.setPurpose(eSerialsList.get(t).getPurpose());
                                        //分红性质类别
                                        abcExcie.setBonus(eSerialsList.get(t).getBonus());
                                        //其他交易性质
                                        abcExcie.setTrade_nature(eSerialsList.get(t).getTrade_nature());
                                        //审计备注列1
                                        abcExcie.setTemp_f(eSerialsList.get(t).getTemp_f());
                                        //审计备注列2
                                        abcExcie.setTemp_g(eSerialsList.get(t).getTemp_g());
                                        //对端户名
                                        abcExcie.setEnd_account_name(eSerialsList.get(t).getEnd_account_name());
                                        //对端卡号
                                        abcExcie.setEnd_card_no(eSerialsList.get(t).getEnd_card_no());
                                        //对端开户行
                                        abcExcie.setEnd_bank_name(eSerialsList.get(t).getEnd_bank_name());
                                        //交易摘要
                                        abcExcie.setRemark(eSerialsList.get(t).getRemark());
                                        //交易备注
                                        abcExcie.setComment(eSerialsList.get(t).getComment());
                                        //交易场所
                                        abcExcie.setTreade_place(eSerialsList.get(t).getTreade_place());
                                        eSerialsList1.add(abcExcie);
                                    }
                                }
                                transferRelationshipService.insertTransferRelationship(eSerialsList1);
                                quantityPerServing=quantityPerServing+1000;
                        }
                    }
                }).sheet().doRead();//sheet()参数指定，默认读取第一张工作表
        return new Result(ResultCode.SUCCESS);
    }


}
