package com.example.gajz.transferrelationship.service.serviceImpl;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.example.gajz.redis.ConFigRedis;
import com.example.gajz.result.Result;
import com.example.gajz.result.ResultCode;
import com.example.gajz.transferrelationship.bean.dto.*;
import com.example.gajz.transferrelationship.bean.echarts.lineStyle;
import com.example.gajz.transferrelationship.bean.excel.abcExcie;
import com.example.gajz.transferrelationship.bean.param.DiagramSpotParam;
import com.example.gajz.transferrelationship.bean.param.TransferAssociationParam;
import com.example.gajz.transferrelationship.bean.vo.*;
import com.example.gajz.transferrelationship.config.Colour;
import com.example.gajz.transferrelationship.dao.TransferRelationshipMapper;
import com.example.gajz.transferrelationship.relationalalgorithm.*;
import com.example.gajz.transferrelationship.service.TransferRelationshipService;
import org.apache.commons.lang.StringUtils;
import org.checkerframework.checker.units.qual.A;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.annotation.RequestParam;
import redis.clients.jedis.Jedis;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;


/**
 * @author lenovo
 */
@Service
public class TransferRelationshipServiceImpl implements TransferRelationshipService {

    /**关系图 数据持久层*/
    @Autowired
    TransferRelationshipMapper transferRelationshipMapper;

    /**计算中心点位置工具类*/
    @Autowired
    CalculationCenterPoint calculationCenterPoint;

    /**关系图 - 分散图形算法工具类*/
    @Autowired
    DispersionDiagram dispersionDiagram;

    /**关系图 - 圆形图形算法工具类*/
    @Autowired
    CircularAlgorithm circularAlgorithm;


    /**
     * 转账关系
     * @return
     */
    @Override
    public List<com.example.gajz.transferrelationship.bean.param.data> edge() {
        List<com.example.gajz.transferrelationship.bean.param.data> edge = transferRelationshipMapper.edge();
        List transList=new ArrayList<>();
        for (int i=0;i<edge.size();i++){
            TransferRelationship transferRelationship=new TransferRelationship();
            Source source=new Source();
            source.setId(edge.get(i).getSource());
            Target target=new Target();
            target.setId(edge.get(i).getTarget());
            /**
             * 转账次数
             */
            TransferTimes transferTimes=new TransferTimes();
            if (edge.get(i).getTransferTimes()>2){
                transferTimes.setLineSize(1);
            }else {
                transferTimes.setLineSize(0.5);
            }
            transferTimes.setTransferTimes(edge.get(i).getTransferTimes());
            transferRelationship.setSource(source);
            transferRelationship.setTarget(target);
            transferRelationship.setTransferTimes(transferTimes);
            transList.add(transferRelationship);
        }
        return transList;
    }



    /**
     * 点
     */
    @Override
    public List diagramSpot(DiagramSpotParam diagramSpotParam) {
          if (diagramSpotParam.getDrawingNumber()==0){   /**分散图形算法 */
               List list = dispersionDiagram.dispersionDiagram(diagramSpotParam.getNumerical());
              return list;
          }else if (diagramSpotParam.getDrawingNumber()==1){ /**圆形图形算法*/
              List list = circularAlgorithm.circularAlgorithm(diagramSpotParam.getNumerical());
              return list;
          }
        return null;
    }

    /**
     * 关系图搜索 单人转账关系（生成点）接口   （二级页面）
     * @param cardnumbe
     * @return
     */
    @Override
    public Result searchRelationship(String cardnumbe) {
//        try {
            //查询搜索人信息   转账人
            List<SearchRelationshipVo> searchRelationshipVos1 = transferRelationshipMapper.personalInformationTransferor(cardnumbe);
            //查询搜索人信息   被转账人
            List<SearchRelationshipVo> searchRelationshipVos2 = transferRelationshipMapper.personalInformationTransferredPerson(cardnumbe);
            /**先查询用户信息是转账人还是被转账人  之后将数据存入list中 如果两个都没有就返回null*/
            List<SearchRelationshipVo> searchRelationshipVos = new ArrayList<>();
            if (searchRelationshipVos1.size()!=0||searchRelationshipVos2.size()!=0){
                if (searchRelationshipVos1.size()!=0){
                    for (int i=0;i<searchRelationshipVos1.size();i++) {
                        SearchRelationshipVo searchRelationshipVo = new SearchRelationshipVo();
                        searchRelationshipVo.setName(searchRelationshipVos1.get(i).getName());
                        searchRelationshipVo.setRelationship(searchRelationshipVos1.get(i).getRelationship());
                        searchRelationshipVo.setTransferTimes(searchRelationshipVos1.get(i).getTransferTimes());
                        searchRelationshipVo.setCardNumber(searchRelationshipVos1.get(i).getCardNumber());
                        searchRelationshipVos.add(searchRelationshipVo);
                    }
                }else{
                    for (int i=0;i<searchRelationshipVos2.size();i++) {
                        SearchRelationshipVo searchRelationshipVo = new SearchRelationshipVo();
                        searchRelationshipVo.setName(searchRelationshipVos2.get(i).getName());
                        searchRelationshipVo.setRelationship(searchRelationshipVos2.get(i).getRelationship());
                        searchRelationshipVo.setTransferTimes(searchRelationshipVos2.get(i).getTransferTimes());
                        searchRelationshipVo.setCardNumber(searchRelationshipVos2.get(i).getCardNumber());
                        searchRelationshipVos.add(searchRelationshipVo);
                    }
                }
            }else {
                return new Result(ResultCode.SUCCESS);
            }
            //存放数据list
            List<SearchRelationshipVo> searchRelationshipVoList = new ArrayList<>();
            /**创建map 将已经查询出来的下级存入map中  防止相互转账进入死循环*/
            HashMap hashMap=new HashMap();
            //转账人
            for (int i = 0; i < searchRelationshipVos.size(); i++) {
                SearchRelationshipVo searchRelationshipVo = new SearchRelationshipVo();
                searchRelationshipVo.setName(searchRelationshipVos.get(i).getName());
                searchRelationshipVo.setRelationship(searchRelationshipVos.get(i).getRelationship());
                searchRelationshipVo.setTransferTimes(searchRelationshipVos.get(i).getTransferTimes());
                searchRelationshipVo.setCardNumber(searchRelationshipVos.get(i).getCardNumber());
                //存入map
                hashMap.put(searchRelationshipVos.get(i).getCardNumber(),searchRelationshipVos.get(i).getCardNumber());
                /** 查询下级数据*/
                List<SearchRelationshipVo> searchRelationshipVoSubordinateList = transferRelationshipMapper.querySubordinate(searchRelationshipVos.get(i).getCardNumber());
                /**判断是否有下级  如果有给true*/
                if (searchRelationshipVoSubordinateList.size()!=0){
                    searchRelationshipVo.setSubordinate(true);
                }else {
                    searchRelationshipVo.setSubordinate(false);
                }
                searchRelationshipVoList.add(searchRelationshipVo);
                if (searchRelationshipVoSubordinateList.size() != 0) {
                    /**存放下级参数list*/
                    List<SearchRelationshipVo> list = new ArrayList<>();
                    for (int n = 0; n < searchRelationshipVoSubordinateList.size(); n++) {
                        SearchRelationshipVo searchRelationshipSubordinateVo = new SearchRelationshipVo();
                        searchRelationshipSubordinateVo.setName(searchRelationshipVoSubordinateList.get(n).getName());
                        searchRelationshipSubordinateVo.setCardNumber(searchRelationshipVoSubordinateList.get(n).getCardNumber());
                        searchRelationshipSubordinateVo.setTransferTimes(searchRelationshipVoSubordinateList.get(n).getTransferTimes());
                        searchRelationshipSubordinateVo.setRelationship(searchRelationshipVos.get(i).getCardNumber());
                        List<SearchRelationshipVo> searchRelationshipVoSubordinateList1 = transferRelationshipMapper.querySubordinate(searchRelationshipVoSubordinateList.get(n).getCardNumber());
                        /**判断是否有下级  如果有给true*/
                        if (searchRelationshipVoSubordinateList1.size()!=0){
                            searchRelationshipSubordinateVo.setSubordinate(true);
                        }else {
                            searchRelationshipSubordinateVo.setSubordinate(false);
                        }
                        //存放数据
                        searchRelationshipVoList.add(searchRelationshipSubordinateVo);
                        //存入map
                        hashMap.put(searchRelationshipVoSubordinateList.get(n).getCardNumber(),searchRelationshipVoSubordinateList.get(n).getCardNumber());
                        list.add(searchRelationshipSubordinateVo);
                    }
                    for (int r = 0; r < 1; ) {
                        //临时存储查询出来的参数
                        List<SearchRelationshipVo> list1 = new ArrayList<>();
                        for (int y = 0; y < list.size(); y++) {
                            List<SearchRelationshipVo> searchRelationshipVoSubordinateList1 = transferRelationshipMapper.querySubordinate(list.get(y).getCardNumber());
                            if (searchRelationshipVoSubordinateList1.size() != 0) {
                                for (int l=0;l<searchRelationshipVoSubordinateList1.size();l++) {
                                    /**根据下级卡号 对比map 如果已经有值代表已经查询出过此卡号 跳过此次循环开启下一圈*/
                                    Object number = hashMap.get(searchRelationshipVoSubordinateList1.get(l).getCardNumber());
                                    if (number==null) {
                                        SearchRelationshipVo searchRelationshipSubordinateVo = new SearchRelationshipVo();
                                        searchRelationshipSubordinateVo.setName(searchRelationshipVoSubordinateList1.get(l).getName());
                                        searchRelationshipSubordinateVo.setCardNumber(searchRelationshipVoSubordinateList1.get(l).getCardNumber());
                                        searchRelationshipSubordinateVo.setRelationship(list.get(y).getCardNumber());
                                        searchRelationshipSubordinateVo.setTransferTimes(searchRelationshipVoSubordinateList1.get(l).getTransferTimes());
                                        List<SearchRelationshipVo> searchRelationshipVoSubordinateList2 = transferRelationshipMapper.querySubordinate(searchRelationshipVoSubordinateList1.get(l).getCardNumber());
                                        if (searchRelationshipVoSubordinateList2.size()!=0){
                                            searchRelationshipSubordinateVo.setSubordinate(true);
                                        }else {
                                            searchRelationshipSubordinateVo.setSubordinate(false);
                                        }
                                        searchRelationshipVoList.add(searchRelationshipSubordinateVo);
                                        list1.add(searchRelationshipSubordinateVo);
                                        //存入map
                                        hashMap.put(searchRelationshipVoSubordinateList1.get(l).getCardNumber(), searchRelationshipVoSubordinateList1.get(l).getCardNumber());
                                    }
                                }
                            }
                        }
                        //清空上一级的参数
                        list.clear();
                        if (list1.size() == 0) {
                            r++;
                        }
                        //将新查询出来下一级的参数放入list当中
                        for (int v = 0; v < list1.size(); v++) {
                            /**循环完成后  判断只让有下级的人进入下次循环*/
                            if (list1.get(v).getSubordinate()){
                                SearchRelationshipVo searchRelationshipSubordinateVo = new SearchRelationshipVo();
                                searchRelationshipSubordinateVo.setCardNumber(list1.get(v).getCardNumber());
                                list.add(searchRelationshipSubordinateVo);
                            }
                        }
                        list1.clear();

                    }
                }
            }
            RelationalAlgorithm relationalAlgorithm=new RelationalAlgorithm();
            //生成Redis随机数
            int intFlag = (int)(Math.random() * 1000000);
            //开启redis
            Jedis redis = ConFigRedis.getConn();
            int o=0;
            for (int i=0;i<searchRelationshipVoList.size();i++){
                if (searchRelationshipVoList.get(i).getSubordinate()){
                    redis.set(intFlag+"cardnumbe"+o++,searchRelationshipVoList.get(i).getCardNumber());
                }
            }
            //关闭redis
            redis.close();
            List list = relationalAlgorithm.searchPresentation(searchRelationshipVoList,intFlag);
            return new Result(ResultCode.SUCCESS,list);
//        }catch (Exception e){
//            return new Result(ResultCode.FAIL);
//        }
    }

    /**
     * 转账关系图 信息搜索接口  查询某一个点的具体位置
     * @param id
     * @return
     */
    @Override
    public Result specificLocation(String id) {
        try {
            EchartsUserVo echartsUserVo=new EchartsUserVo();
            Jedis conn = ConFigRedis.getConn();
            String position = conn.get(id);
            /**
             * 截取X
             */
            String x = StringUtils.substringBefore(position, "Y");
            /**
             * 截取Y
             */
            String Y = position.substring(x.length()+1, position.length());
            echartsUserVo.setName(id);
            echartsUserVo.setX(Integer.parseInt(x));
            echartsUserVo.setY(Integer.parseInt(Y));
            return new Result(ResultCode.SUCCESS,echartsUserVo);
        }catch (Exception e){
            System.out.println(e);
            return new Result(ResultCode.FAIL);
        }
    }

    /**
     * 清空关系图数据表
     */
    @Override
    public void emptyTransferRelationship() {
        transferRelationshipMapper.emptyTransferRelationship();
    }

    /**
     * excel数据导入数据库
     * @param eSerialsList
     */
    @Override
    public void insertTransferRelationship(List<abcExcie> eSerialsList) {
        transferRelationshipMapper.insertTransferRelationship(eSerialsList);
    }

    /**
     * 关系图 搜索接口
     * @param name
     * @return
     */
    @Override
    public Result pointSearch(@RequestParam("name") String name) {
        try {
            boolean numeric = StringUtils.isNumeric(name);
            String cardNumber = null;
            String fullName = null;
            if (numeric) {
                cardNumber = name;
            } else {
                fullName = name;
            }
            //查询转账人
            List<SearchRelationshipVo> searchRelationshipVos = this.transferRelationshipMapper.queryTransferor(cardNumber, fullName);
            //查询被转账人
            List<SearchRelationshipVo> searchRelationshipVos1 = this.transferRelationshipMapper.queryTheTransferredPerson(cardNumber, fullName);
            List<SearchRelationshipVo> searchRelationshipVoList = new ArrayList<>();
            HashMap hashMap=new HashMap();
            if (searchRelationshipVos.size() != 0 || searchRelationshipVos1.size() != 0 ) {
                if (searchRelationshipVos.size() != 0) {
                    for (int i = 0; i < searchRelationshipVos.size(); i++) {
                        if (hashMap.get(searchRelationshipVos.get(i).getCardNumber())==null){
                            SearchRelationshipVo searchRelationshipVo = new SearchRelationshipVo();
                            searchRelationshipVo.setName(searchRelationshipVos.get(i).getName());
                            searchRelationshipVo.setRelationship(searchRelationshipVos.get(i).getRelationship());
                            searchRelationshipVo.setCardNumber(searchRelationshipVos.get(i).getCardNumber());
                            searchRelationshipVoList.add(searchRelationshipVo);
                            hashMap.put(searchRelationshipVos.get(i).getCardNumber(),searchRelationshipVos.get(i).getCardNumber());
                        }
                    }
                }
                if (searchRelationshipVos1.size() != 0) {
                    for (int n = 0; n < searchRelationshipVos1.size(); n++) {
                        if (hashMap.get(searchRelationshipVos1.get(n).getCardNumber())==null) {
                            SearchRelationshipVo searchRelationshipVo = new SearchRelationshipVo();
                            searchRelationshipVo.setName(searchRelationshipVos1.get(n).getName());
                            searchRelationshipVo.setRelationship(searchRelationshipVos1.get(n).getRelationship());
                            searchRelationshipVo.setCardNumber(searchRelationshipVos1.get(n).getCardNumber());
                            searchRelationshipVoList.add(searchRelationshipVo);
                        }
                    }
                }
                return new Result(ResultCode.SUCCESS, searchRelationshipVoList);
            }
            if (searchRelationshipVos.size() != 0)
                return new Result(ResultCode.SUCCESS, searchRelationshipVos);
            return new Result(ResultCode.SUCCESS, searchRelationshipVos1);
        } catch (Exception e) {
            return new Result(ResultCode.FAIL);
        }
    }


    /**
     * 查询两人转账关系  (生成点)(关系图) 接口
     * @param transferAssociationParam
     * @return
     */
    @Override
    public Result transferAssociation(TransferAssociationParam transferAssociationParam) {
        //点 X  Y轴生成类
        TransferLocation transferLocation=new TransferLocation();
        //层级
        int Hierarchy=1;
        //生成颜色
        Colour colour=new Colour();
        //查询搜索人信息   转账人
        List<SearchRelationshipVo> searchRelationshipVos = transferRelationshipMapper.personalInformationTransferor(transferAssociationParam.getTransfer());
        //查询搜索人信息   被转账人
        List<SearchRelationshipVo> searchRelationshipVos1 = transferRelationshipMapper.personalInformationTransferredPerson(transferAssociationParam.getTransfer());
        //将查询出来的转账人数据放入list中
        List<SearchRelationshipVo> searchRelationshipVoList1 = new ArrayList<>();
        SearchRelationshipVo searchRelationshipVo1 = new SearchRelationshipVo();
        /** 创建map 存放已经查询到的上级卡号    用于判断卡号是否已经重复查询*/
        HashMap hashMap1=new HashMap();
        //判断 是转账人列查询出来的 还是被转账人列查询出来的
        if (searchRelationshipVos.size() != 0) {
            for (int i = 0; i < searchRelationshipVos.size(); i++) {
                searchRelationshipVo1.setCardNumber(searchRelationshipVos.get(i).getCardNumber());
                searchRelationshipVo1.setName(searchRelationshipVos.get(i).getName());
                searchRelationshipVo1.setRelationship(searchRelationshipVos.get(i).getRelationship());
                searchRelationshipVo1.setSubordinate(searchRelationshipVos.get(i).getSubordinate());
                searchRelationshipVo1.setTransferTimes(searchRelationshipVos.get(i).getTransferTimes());
                searchRelationshipVoList1.add(searchRelationshipVo1);
            }
        } else {
            for (int i = 0; i < searchRelationshipVos1.size(); i++) {
                searchRelationshipVo1.setCardNumber(searchRelationshipVos1.get(i).getCardNumber());
                searchRelationshipVo1.setName(searchRelationshipVos1.get(i).getName());
                searchRelationshipVo1.setRelationship(searchRelationshipVos1.get(i).getRelationship());
                searchRelationshipVo1.setSubordinate(searchRelationshipVos1.get(i).getSubordinate());
                searchRelationshipVo1.setTransferTimes(searchRelationshipVos1.get(i).getTransferTimes());
                searchRelationshipVoList1.add(searchRelationshipVo1);
            }
        }
        /**将有关系的数据放入这个list中  最终返回这个list*/
        List<SearchRelationshipVo> transferAssociationList = new ArrayList<>();
        /** 存放下级关系数据*/
        List<SearchRelationshipVo> searchRelationshipVoList = new ArrayList<>();
        /**创建map 将已经查询出来的下级存入map中  防止相互转账进入死循环*/
        HashMap hashMap = new HashMap();
        //转账人
        for (int i = 0; i < searchRelationshipVoList1.size(); i++) {
            SearchRelationshipVo searchRelationshipVo = new SearchRelationshipVo();
            searchRelationshipVo.setName(searchRelationshipVoList1.get(i).getName());
            searchRelationshipVo.setRelationship(searchRelationshipVoList1.get(i).getRelationship());
            searchRelationshipVo.setTransferTimes(searchRelationshipVoList1.get(i).getTransferTimes());
            searchRelationshipVo.setX(-300);
            searchRelationshipVo.setY(600);
            searchRelationshipVo.setColor(colour.getColour());
            searchRelationshipVo.setSize(20);
            searchRelationshipVo.setHierarchy(Hierarchy);
            searchRelationshipVo.setCardNumber(searchRelationshipVoList1.get(i).getCardNumber());
            //层级递增
            Hierarchy++;
            //存入map
            hashMap.put(searchRelationshipVoList1.get(i).getCardNumber(), searchRelationshipVoList1.get(i).getCardNumber());
            /** 查询下级数据*/
            List<SearchRelationshipVo> searchRelationshipVoSubordinateList = transferRelationshipMapper.querySubordinate(searchRelationshipVoList1.get(i).getCardNumber());
            searchRelationshipVo.setSubordinateQuantity(searchRelationshipVoSubordinateList.size());
            /**判断是否有下级  如果有给true*/
            if (searchRelationshipVoSubordinateList.size() != 0) {
                searchRelationshipVo.setSubordinate(true);
            } else {
                searchRelationshipVo.setSubordinate(false);
            }
            searchRelationshipVoList.add(searchRelationshipVo);
            transferAssociationList.add(searchRelationshipVo);
            if (searchRelationshipVoSubordinateList.size() != 0) {
                /**存放下级参数list*/
                List<SearchRelationshipVo> list = new ArrayList<>();
                for (int n = 0; n < searchRelationshipVoSubordinateList.size(); n++) {
                    SearchRelationshipVo searchRelationshipSubordinateVo = new SearchRelationshipVo();
                    searchRelationshipSubordinateVo.setName(searchRelationshipVoSubordinateList.get(n).getName());
                    searchRelationshipSubordinateVo.setCardNumber(searchRelationshipVoSubordinateList.get(n).getCardNumber());
                    searchRelationshipSubordinateVo.setTransferTimes(searchRelationshipVoSubordinateList.get(n).getTransferTimes());
                    searchRelationshipSubordinateVo.setRelationship(searchRelationshipVos.get(i).getCardNumber());
                    searchRelationshipSubordinateVo.setHierarchy(Hierarchy);
                    //查询下级
                    List<SearchRelationshipVo> searchRelationshipVoSubordinateList1 = transferRelationshipMapper.querySubordinate(searchRelationshipVoSubordinateList.get(n).getCardNumber());
                    searchRelationshipSubordinateVo.setSubordinateQuantity(searchRelationshipVoSubordinateList1.size());
                    /**判断这一层中是否有被转账人 如果有就存入list中 结束循环*/
                    if (searchRelationshipVoSubordinateList.get(n).getCardNumber().equals(transferAssociationParam.getPayee())) {
                        searchRelationshipSubordinateVo.setSubordinate(false);
                        searchRelationshipSubordinateVo.setX((int)(Math.random()*2000));
                        searchRelationshipSubordinateVo.setY((int)(Math.random()*800));
                        searchRelationshipSubordinateVo.setColor(colour.getColour());
                        searchRelationshipSubordinateVo.setSize(10);
                        transferAssociationList.add(searchRelationshipSubordinateVo);
                        List<SearchRelationshipVo> searchRelationshipVoList2 = transferLocation.transferLocation(transferAssociationList);
                        return new Result(ResultCode.SUCCESS, searchRelationshipVoList2);
                    }
                    /**判断是否有下级  如果有给true*/
                    if (searchRelationshipVoSubordinateList1.size() != 0) {
                        searchRelationshipSubordinateVo.setSubordinate(true);
                    } else {
                        searchRelationshipSubordinateVo.setSubordinate(false);
                    }
                    //存放数据
                    searchRelationshipVoList.add(searchRelationshipSubordinateVo);
                    //存入map
                    hashMap.put(searchRelationshipVoSubordinateList.get(n).getCardNumber(), searchRelationshipVoSubordinateList.get(n).getCardNumber());
                    list.add(searchRelationshipSubordinateVo);
                }
                for (int r = 0; r < 1; ) {
                    Hierarchy++;
                    //临时存储查询出来的参数
                    List<SearchRelationshipVo> list1 = new ArrayList<>();
                    for (int y = 0; y < list.size(); y++) {
                        //查询下级数据
                        List<SearchRelationshipVo> searchRelationshipVoSubordinateList1 = transferRelationshipMapper.querySubordinate(list.get(y).getCardNumber());
                        if (searchRelationshipVoSubordinateList1.size() != 0) {
                            for (int l = 0; l < searchRelationshipVoSubordinateList1.size(); l++) {
                                /**根据下级卡号 对比map 如果已经有值代表已经查询出过此卡号 跳过此次循环开启下一圈*/
                                Object number = hashMap.get(searchRelationshipVoSubordinateList1.get(l).getCardNumber());
                                if (number == null) {
                                    SearchRelationshipVo searchRelationshipSubordinateVo = new SearchRelationshipVo();
                                    searchRelationshipSubordinateVo.setSubordinateQuantity(searchRelationshipVoSubordinateList1.size());
                                    searchRelationshipSubordinateVo.setName(searchRelationshipVoSubordinateList1.get(l).getName());
                                    searchRelationshipSubordinateVo.setCardNumber(searchRelationshipVoSubordinateList1.get(l).getCardNumber());
                                    searchRelationshipSubordinateVo.setRelationship(list.get(y).getCardNumber());
                                    searchRelationshipSubordinateVo.setTransferTimes(searchRelationshipVoSubordinateList1.get(l).getTransferTimes());
                                    searchRelationshipSubordinateVo.setHierarchy(Hierarchy);
                                    List<SearchRelationshipVo> searchRelationshipVoSubordinateList2 = transferRelationshipMapper.querySubordinate(searchRelationshipVoSubordinateList1.get(l).getCardNumber());
                                    searchRelationshipSubordinateVo.setSubordinateQuantity(searchRelationshipVoSubordinateList2.size());
                                    if (searchRelationshipVoSubordinateList2.size() != 0) {
                                        searchRelationshipSubordinateVo.setSubordinate(true);
                                    } else {
                                        searchRelationshipSubordinateVo.setSubordinate(false);
                                    }
                                    searchRelationshipVoList.add(searchRelationshipSubordinateVo);
                                    list1.add(searchRelationshipSubordinateVo);
                                    //存入map
                                    hashMap.put(searchRelationshipVoSubordinateList1.get(l).getCardNumber(), searchRelationshipVoSubordinateList1.get(l).getCardNumber());
                                    /**
                                     * 判断这一层中是否有被转账人 如果有就存入返回值list中   如果进入此判断 代表已经找到被转账人  之后从被转账人往上循环推进
                                     */
                                    if (searchRelationshipSubordinateVo.getCardNumber().equals(transferAssociationParam.getPayee())) {
                                        searchRelationshipSubordinateVo.setX(2300);
                                        searchRelationshipSubordinateVo.setY(600);
                                        searchRelationshipSubordinateVo.setColor(colour.getColour());
                                        searchRelationshipSubordinateVo.setSize(20);
                                        searchRelationshipSubordinateVo.setHierarchy(Hierarchy);
                                        //添加被转账人信息
                                        transferAssociationList.add(searchRelationshipSubordinateVo);
                                        //存放需要查询上级的人的卡号  List
                                        List<SearchRelationshipVo> superiorList = new ArrayList<>();
                                        superiorList.add(searchRelationshipSubordinateVo);
                                        //向防止重复map添加已经查询到的上级卡号
                                        hashMap1.put(searchRelationshipSubordinateVo.getCardNumber(),searchRelationshipSubordinateVo.getCardNumber());
                                        for (int b=0;b<1;) {
                                            for (int a = 0; a < superiorList.size(); a++) {

                                                //存放查询出来的上级信息
                                                List<SearchRelationshipVo> superior=new ArrayList<>();
                                                //循环查询出来的上级信息  之后将查出来的上级信息存入到list中
                                                for (int m=0;m<superiorList.size();m++){
                                                    //查询上级转账人方法
                                                    List<SearchRelationshipVo> superior1 = transferRelationshipMapper.querySuperior(superiorList.get(m).getCardNumber());
                                                    for (int j=0;j<superior1.size();j++){
                                                        superior.add(superior1.get(j));
                                                    }
                                                }
                                                //清空list数据
                                                superiorList.clear();
                                                //判断上级是否是转账人  如果是 就直接返回数据
                                                for (int h = 0; h < superior.size(); h++) {
                                                    if (superior.get(h).getCardNumber().equals(transferAssociationParam.getTransfer())) {
                                                        List<SearchRelationshipVo> searchRelationshipVoList2 = transferLocation.transferLocation(transferAssociationList);
                                                        return new Result(ResultCode.SUCCESS, searchRelationshipVoList2);
                                                    }
                                                }
                                                //循环对比查询出来的上级与所有list存储的下级做对比
                                                for (int k = 0; k < searchRelationshipVoList.size(); k++) {
                                                    for (int n = 0; n < superior.size(); n++) {
                                                        //对比
                                                        if (superior.get(n).getCardNumber().equals(searchRelationshipVoList.get(k).getCardNumber())) {
                                                            SearchRelationshipVo searchRelationshipSubordinateVo1 = new SearchRelationshipVo();
                                                            searchRelationshipSubordinateVo1.setCardNumber(superior.get(n).getCardNumber());
                                                            searchRelationshipSubordinateVo1.setName(superior.get(n).getName());
                                                            searchRelationshipSubordinateVo1.setX((int) (Math.random() * 2000));
                                                            searchRelationshipSubordinateVo1.setY((int) (Math.random() * 800));
                                                            searchRelationshipSubordinateVo1.setColor(colour.getColour());
                                                            searchRelationshipSubordinateVo1.setSize(10);
                                                            searchRelationshipSubordinateVo1.setHierarchy(searchRelationshipVoList.get(k).getHierarchy());
                                                            searchRelationshipSubordinateVo1.setSubordinateQuantity(searchRelationshipVoList.get(k).getSubordinateQuantity());
                                                            Object Number = hashMap1.get(superior.get(n).getCardNumber());
                                                            if (Number==null){
                                                                //添加被转账人信息
                                                                transferAssociationList.add(searchRelationshipSubordinateVo1);
                                                                //向防止重复map添加已经查询到的上级卡号
                                                                hashMap1.put(superior.get(n).getCardNumber(), superior.get(n).getCardNumber());
                                                            }
                                                            superiorList.add(searchRelationshipSubordinateVo1);
                                                        }
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                    //清空上一级的参数
                    list.clear();
                    if (list1.size() == 0) {
                        r++;
                    }
                    //将新查询出来下一级的参数放入list当中
                    for (int v = 0; v < list1.size(); v++) {
                        /**循环完成后  判断只让有下级的人进入下次循环*/
                        if (list1.get(v).getSubordinate()) {
                            SearchRelationshipVo searchRelationshipSubordinateVo = new SearchRelationshipVo();
                            searchRelationshipSubordinateVo.setCardNumber(list1.get(v).getCardNumber());
                            list.add(searchRelationshipSubordinateVo);
                        }
                    }
                    list1.clear();
                }
            }
        }
        for (int p = 0; p < searchRelationshipVoList.size(); p++) {
            if (searchRelationshipVoList.get(p).getCardNumber() == transferAssociationParam.getPayee()) {
                List<SearchRelationshipVo> searchRelationshipVoList2 = transferLocation.transferLocation(searchRelationshipVoList);
                return new Result(ResultCode.SUCCESS, searchRelationshipVoList2);
            }
        }
        //如果没有 就返回null
        return new Result(ResultCode.SUCCESS, null);
    }


    /**
     * 个人关系图 （转账关系） 搜索接口  (二级页面)
     * @return
     */
    @Override
    public Result personalTransferRelationship(int RedisId){
        Jedis redis = ConFigRedis.getConn();
        List list=new ArrayList();
        for (int i=0;i<i+1;i++){
            //获取redis
            String  cardnumbe= redis.get(RedisId+"cardnumbe" + i);
            //删除redis kye
            redis.del(String.valueOf(RedisId+"cardnumbe" + i));
            if (cardnumbe!=null){
                /**
                 * 查询单人转账关系
                 */
                List<personalTransferRelationshipVo> personalTransferRelationshipVos = transferRelationshipMapper.personalTransferRelationship(cardnumbe);
                for (int n=0;n<personalTransferRelationshipVos.size();n++){
                    TransferRelationship transferRelationship=new TransferRelationship();
                    Source source=new Source();
                    source.setId(personalTransferRelationshipVos.get(n).getSource());
                    Target target=new Target();
                    target.setId(personalTransferRelationshipVos.get(n).getTarget());
                    TransferTimes transferTimes=new TransferTimes();
                    transferTimes.setTransferTimes(personalTransferRelationshipVos.get(n).getTransferTimes());
                    transferRelationship.setSource(source);
                    transferRelationship.setTarget(target);
                    transferRelationship.setTransferTimes(transferTimes);
                    list.add(transferRelationship);
                }
            }else {
                redis.close();
                return new Result(ResultCode.SUCCESS,list);
            }
        }
        redis.close();
        return new Result(ResultCode.SUCCESS);
    }


    /**
     * 查询两人转账关系  关系  (关系图) 接口
     * @param transferAssociationParam
     * @return
     */
    @Override
    public Result trBetweenTwoPersons(List<SearchRelationshipVo> transferAssociationParam) {
        List<TransferRelationship> list=new ArrayList();
        for (int i=0;i<transferAssociationParam.size();i++){
            if (!"个人".equals(transferAssociationParam.get(i).getRelationship())){
                List<personalTransferRelationshipVo> personalTransferRelationshipVos = transferRelationshipMapper.trBetweenTwoPersons(transferAssociationParam.get(i).getCardNumber());
                for (int n=0;n<personalTransferRelationshipVos.size();n++){
                    TransferRelationship transferRelationship=new TransferRelationship();
                    Source source=new Source();
                    source.setId(personalTransferRelationshipVos.get(n).getSource());
                    Target target=new Target();
                    target.setId(personalTransferRelationshipVos.get(n).getTarget());
                    TransferTimes transferTimes=new TransferTimes();
                    transferTimes.setTransferTimes(personalTransferRelationshipVos.get(n).getTransferTimes());
                    transferRelationship.setSource(source);
                    transferRelationship.setTarget(target);
                    transferRelationship.setTransferTimes(transferTimes);
                    if (!personalTransferRelationshipVos.get(n).getSource().equals(personalTransferRelationshipVos.get(n).getTarget())){
                        list.add(transferRelationship);
                    }
                }
            }
        }
        List<TransferRelationship> transferRelationshipList=new ArrayList();
        for (int k=0;k<list.size();k++){
            TransferRelationship transferRelationship=new TransferRelationship();
            for (int n=0;n<transferAssociationParam.size();n++){
                if (list.get(k).getSource().getId().equals(transferAssociationParam.get(n).getCardNumber())){
                    Source source=new Source();
                    String o= String.valueOf(n);
                    source.setId(o);
                    transferRelationship.setSource(source);
                }
                if (list.get(k).getTarget().getId().equals(transferAssociationParam.get(n).getCardNumber())){
                    Target target=new Target();
                    String o= String.valueOf(n);
                    target.setId(o);
                    transferRelationship.setTarget(target);
                }
            }
            transferRelationship.setTransferTimes(list.get(k).getTransferTimes());
            transferRelationshipList.add(transferRelationship);
        }

        return new Result(ResultCode.SUCCESS,list);
    }

    /**
     * echarts 双人转账关系图 点  生成接口
     * @param transferAssociationParam
     * @return
     */
    @Override
    public Result echartsTransferAssociation(TransferAssociationParam transferAssociationParam) {
        Result result = this.transferAssociation(transferAssociationParam);
        Object obj = result.getData();
        /**将有关系的数据放入这个list中 */
        List<SearchRelationshipVo> transferAssociationList = new ArrayList<>();
        if (obj instanceof ArrayList<?>) {
            for (Object o : (List<?>) obj) {
                transferAssociationList.add(SearchRelationshipVo.class.cast(o));
            }
        }

        List<EchartsUserVo> list=new ArrayList<>();
        for (int i=0;i<transferAssociationList.size();i++){
            EchartsUserVo echartsUserVo=new EchartsUserVo();
            echartsUserVo.setY(transferAssociationList.get(i).getY());
            echartsUserVo.setX(transferAssociationList.get(i).getX());
            echartsUserVo.setName(transferAssociationList.get(i).getCardNumber());
            list.add(echartsUserVo);
        }
        return new Result(ResultCode.SUCCESS,list);
    }

    /**
     * echarts 双人转账关系图 关系 生成接口
     * @param transferAssociationParam
     * @return
     */
    @Override
    public Result echartsTrBetweenTwoPersons(List<SearchRelationshipVo> transferAssociationParam) {
        Result result = this.trBetweenTwoPersons(transferAssociationParam);
        Object obj = result.getData();
        /**将有关系的数据放入这个list中 */
        List<TransferRelationship> transferAssociationList = new ArrayList<>();
        if (obj instanceof ArrayList<?>) {
            for (Object o : (List<?>) obj) {
                transferAssociationList.add(TransferRelationship.class.cast(o));
            }
        }
        /**将已经获取到的关系数据放入新list当中返回*/
        List<EchartsTrBetweenTwoPersonsVo> echartsTrBetweenTwoPersonsVosList=new ArrayList<>();
        for (int i=0;i<transferAssociationList.size();i++){
            EchartsTrBetweenTwoPersonsVo echartsTrBetweenTwoPersonsVo=new EchartsTrBetweenTwoPersonsVo();
            echartsTrBetweenTwoPersonsVo.setSource(transferAssociationList.get(i).getSource().getId());
            echartsTrBetweenTwoPersonsVo.setTarget(transferAssociationList.get(i).getTarget().getId());
            echartsTrBetweenTwoPersonsVosList.add(echartsTrBetweenTwoPersonsVo);
        }
        /**计算转账关系曲线*/
        for (int p=0;p<echartsTrBetweenTwoPersonsVosList.size();p++){
            //转账共有几人
            int frequency=0;
            //曲线值
            double curveValue=0.01;
            for (int k=0;k<echartsTrBetweenTwoPersonsVosList.size();k++){
                if (echartsTrBetweenTwoPersonsVosList.get(p).getSource().equals(echartsTrBetweenTwoPersonsVosList.get(k).getSource())){
                    frequency++;
                    if (frequency>2){
                        lineStyle line=new lineStyle();
                        line.setCurveness(curveValue);
                        echartsTrBetweenTwoPersonsVosList.get(p).setLineStyle(line);
                        curveValue=curveValue+0.01;
                    }else {
                        lineStyle line=new lineStyle();
                        line.setCurveness(curveValue);
                        echartsTrBetweenTwoPersonsVosList.get(p).setLineStyle(line);
                    }
                }
            }
        }
        return new Result(ResultCode.SUCCESS,echartsTrBetweenTwoPersonsVosList);
    }
}
